# E.S.O. - VLT project/ ESO Archive
# "@(#) $Id: vmmpsImagePick.tcl,v 3.24 2004/12/18 04:03:45 vltsccm Exp $"
#
# vmmpsImagePick.tcl - widget to define curved slits.
#
# See man page vmmpsImagePick(n) for a complete description.
#
# who              when      what
# --------         --------  ----------------------------------------------
# Dario Bottini   01 Apr 00  Created
# jbrancac        05/03/04   PPRS 11197.Add the control check to DIMX_SLIT and DIMY_SLIT values in check_row method.
# mkiekebu       2004-05-17  added verification of input data

itk::usual vmmpsImagePick {}


# vmmpsImagePick is an itcl widget to select an object (a star, for example) in an 
# image and get statistics for it. It is based on the rtdimage(3) "statistics" 
# subcommand, which uses a centroid algorithm to locate the center of the object.

itcl::class rtd::vmmpsImagePick {
    inherit util::TopLevelWidget


    #  constructor: create a new vmmpsImagePick widget

    constructor {args} {
        eval itk_initialize $args
    }


    # destructor

    destructor {
        catch {cancel}
        catch {$target_image_ view remove $image_}
    }
    

    # This method is called after the options have been evaluated.

    protected method init {} {
	global mode
	if {$mode == "object"} {
 		wm title $w_ "Pick New Object ($itk_option(-number))"
 		wm iconname $w_ "Pick New Object ($itk_option(-number))"
	}
	if {$mode == "slit"} {
 		wm title $w_ "Pick New Slit ($itk_option(-number))"
 		wm iconname $w_ "Pick New Slit ($itk_option(-number))"
	}
	make_layout
	update_scale
	incr initialized_
    }

    
    # This method is responsible for the window layout

    protected method make_layout {} {
        # main controls frame
        pack [set mainf [frame $w_.mainf]] \
            -side top -fill both -expand 1 -padx 1m -pady 1m -ipadx 1m -ipady 1m

        pack [set topf [frame $mainf.topf]] \
            -side top -fill both -expand 1

	if {"$itk_option(-orient)" == "horizontal"} {
	    make_labels $topf left
	    make_rect $topf left
	} else {
	    make_rect $topf top
	    make_labels $topf top
	}
        add_buttons
    }


    # Create the window for displaying the statistics in the given frame.
    # The optional "side" arg is used for packing.

################### Modified for vmmps ###############################
    protected method make_labels {w {side left}} {

        pack [set mylabelf [frame $w.mylabelf -bd 4 -relief groove]] \
        [set mylabelf2 [frame $w.mylabelf2 -bd 4 -relief groove]] \
            -side $side -fill both -expand 0

        # frame for labels
        pack [set labelf [frame $w.labelf -bd 2 -relief groove]] \
            -side $side -fill both -expand 0

        # title for label area
        pack [label $labelf.title -text "Image Statistics:" ] \
            -side top -padx 1m -pady 1m
        
	# LabelValue(n) widget: "Image X" 
        itk_component add x {
            util::LabelValue $labelf.x \
                -text "Image X:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -labelwidth $itk_option(-labelwidth) \
                -valuewidth $itk_option(-valuewidth) \
                -relief groove \
                -anchor e
        } 
	# LabelValue(n) widget: "Image Y"
        itk_component add y {
            util::LabelValue $labelf.y \
                -text "Image Y:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -labelwidth $itk_option(-labelwidth) \
                -valuewidth $itk_option(-valuewidth) \
                -relief groove \
                -anchor e
        } 
	# LabelValue(n) widget for RA (alpha)
        itk_component add ra {
            util::LabelValue $labelf.ra \
                -text "a:" \
                -labelfont $itk_option(-wcsfont) \
                -valuefont $itk_option(-valuefont) \
                -labelwidth $itk_option(-labelwidth) \
                -valuewidth $itk_option(-valuewidth) \
                -relief groove \
                -anchor e
        } 
	# LabelValue(n) widget for DEC (delta)
        itk_component add dec {
            util::LabelValue $labelf.dec \
                -text "d:" \
                -labelfont $itk_option(-wcsfont) \
                -valuefont $itk_option(-valuefont) \
                -labelwidth $itk_option(-labelwidth) \
                -valuewidth $itk_option(-valuewidth) \
                -relief groove \
                -anchor e
        } 
 	# LabelValue(n) widget for equinox
	itk_component add equinox {
            util::LabelValue $labelf.equinox \
                -text "Equinox:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -valuewidth $itk_option(-valuewidth) \
                -labelwidth $itk_option(-labelwidth) \
                -relief groove \
                -anchor e
        } 
	# LabelValue(n) widget for "Peak object level above bg"
        itk_component add object {
            util::LabelValue $labelf.object \
                -text "Peak value above bg:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -valuewidth $itk_option(-valuewidth) \
                -labelwidth $itk_option(-labelwidth) \
                -relief groove \
                -anchor e
        } 
 	# LabelValue(n) widget for "Background level"
	itk_component add background {
            util::LabelValue $labelf.background \
                -text "Background level:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -valuewidth $itk_option(-valuewidth) \
                -labelwidth $itk_option(-labelwidth) \
                -relief groove \
                -anchor e
        } 
	# LabelValue(n) for "FWHM X:Y"
        itk_component add fwhm {
            util::LabelValue $labelf.fwhm \
                -text "FWHM X:Y:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -valuewidth $itk_option(-valuewidth) \
                -labelwidth $itk_option(-labelwidth) \
                -relief groove \
                -anchor e
        } 
 	# LabelValue(n) widget for "Angle of X axis"
	itk_component add angle {
            util::LabelValue $labelf.angle \
                -text "Angle of X axis:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -valuewidth $itk_option(-valuewidth) \
                -labelwidth $itk_option(-labelwidth) \
                -relief groove \
                -anchor e
        } 
 	# LabelValue(n) widget for number of "Pixels in x,y"
        itk_component add nsize {
            util::LabelValue $labelf.nsize \
                -text "Pixels in x,y:" \
                -labelfont $itk_option(-labelfont) \
                -valuefont $itk_option(-valuefont) \
                -valuewidth $itk_option(-valuewidth) \
                -labelwidth $itk_option(-labelwidth) \
                -relief groove \
                -anchor e
        } 

        foreach el {x y ra dec equinox object background fwhm angle nsize} {
            [$itk_component($el) component entry] config -justify right
        }
        pack \
            $itk_component(x) \
            $itk_component(y) \
            $itk_component(ra) \
            $itk_component(dec) \
            $itk_component(equinox) \
            -side top -padx 0.5m -pady 0.0m

#            $itk_component(object) \
#            $itk_component(background) \
#            $itk_component(fwhm) \
#            $itk_component(angle) \
#            $itk_component(nsize) \



        add_short_help $itk_component(x) \
	    {X image pixel coordinate (or X detector chip coord if known)}
        add_short_help $itk_component(y) \
	    {Y Image pixel coordinate (or Y detector chip coord if known)}
        add_short_help $itk_component(ra) \
	    {World Coordinates RA value}
        add_short_help $itk_component(dec) \
	    {World Coordinates DEC value}
        add_short_help $itk_component(equinox) \
	    {World Coordinates equinox (default: J2000)}
        add_short_help $itk_component(object) \
	    {Object: peak value of object above background}
        add_short_help $itk_component(background) \
	    {Background: mean background level}
        add_short_help $itk_component(fwhm) \
	    {FWHM: full width half maximum in X and Y}
        add_short_help $itk_component(angle) \
	    {Angle: angle of major axis, degrees, along X = 0}
        add_short_help $itk_component(nsize) \
	    {Number of pixels: along x and y-axis }
    }


    # Create the window used to display the part of the image being examined.
    # $w is the parent frame.
    # The optional "side" arg is used for packing.

    protected method make_rect {w {side left}} {
        # frame for rect
        pack [set rectf [frame $w.rectf -bd 2 -relief groove]] \
            -side $side -fill both -expand 0

        # add a frame to show the size of the square selected by the scale
        pack [label $rectf.label -text "Area of image to be examined:" ] \
            -side top -padx 1m -pady 1m

        pack [set rf [frame $rectf.rf \
                          -bd 0 -relief flat \
                          -width $itk_option(-maxsize) \
                          -height $itk_option(-maxsize)]] \
            -fill none -expand 1 -anchor c

        # This component displays the section of the image that will be used for
	# the centroid algorithm and statistics.
        itk_component add zoomView {
            RtdImageZoomView $rf.zoomView \
		-target_image $itk_option(-target_image) \
		-verbose $itk_option(-verbose) \
		-width $itk_option(-maxsize) \
		-height $itk_option(-maxsize) \
		-factor $itk_option(-factor) \
		-propagate 0 \
		-usexshm 1 \
		-usexsync 1 \
		-borderwidth 2 \
		-relief groove \
		-shelp {Curved Slit: select an object in the main display and choose some points for the fit in the zoom display}
        }
        set zoomImage [$itk_component(zoomView) component image]
        set image_ [$zoomImage get_image]
        set canvas_ [$zoomImage get_canvas]

        catch {destroy [$itk_component(zoomView) component check]}
        global ::$itk_component(zoomView).dozoom
        set $itk_component(zoomView).dozoom 1
        $itk_component(zoomView) inc_zoom 0

        pack [$itk_component(zoomView) component larger] -expand 1
        pack [$itk_component(zoomView) component smaller] -expand 1
        pack $itk_component(zoomView) -anchor c -fill none -expand 0
        $itk_component(zoomView) inc_zoom 0
    }


    # add a row of dialog buttons at the bottom of the window

####################### Modified for vmmps #########################
    protected method add_buttons {} {

	# dialog buttons frame
        itk_component add buttons {
            frame $w_.buttons -borderwidth 2 -relief groove
        }
        pack $itk_component(buttons) \
            -side top -fill x -padx 1m -pady 1m

	itk_component add pick {
	    button $w_.pick -text "Pick Object" -command [code $this pick_object]
	}
       
	# Enter button
        itk_component add save {
            button $w_.save \
		-text "Enter" \
		-command [code $this save]
        }
	# Close button
        itk_component add close {
            button $w_.close \
		-text "Close" \
		-command [code $this close]
        }

        pack \
	    $itk_component(pick) \
            $itk_component(save) \
            $itk_component(close) \
            -side left -expand 1 -padx 0.5m -pady 2m -in $itk_component(buttons)

	add_short_help $itk_component(pick) \
	    {Pick Object: {bitmap b1} = select object in image and display center and other statistics}
        add_short_help $itk_component(save) {Save: Add new Object in catalog}
        add_short_help $itk_component(close) {Close: close this window}
    }


######################## New method for vmmps ########################
    public method view {} {
        global catname myinfo myresults
        set virg " "
        if {[catch {set obj_id [lindex [lindex $myinfo [$myresults.listbox curselection]] 0]} msg]} {
          error_dialog "You can display only selected and plotted objects"
          $this close
          return
        }
        set n 0
        catch {set n [exec grep -n #arc $catname | cut -f1,3 -d: | cut -f1 -d$virg | grep :$obj_id | cut -f1 -d:]}
        if {$n == 0} {return}
        set params [exec head -$n $catname | tail -1 | cut -f2 -d:]
        set obj_id [lindex $params 0]
        set width [lindex $params 1]
        set dispwidth [expr $width * 0.205]
        $this.mainf.topf.mylabelf.width configure -value [format "%.0f" $dispwidth]

        $canvas_ delete mark
        $target_canvas_ delete mark
        $this plot_slit $obj_id $width [lrange $params 2 end]
        set ax [format "%.7e" [lindex $params 2]]
        set bx [format "%.7e" [lindex $params 3]]
        set cx [format "%.7e" [lindex $params 4]]
        set xx [format "%.7e" [lindex $params 5]]
        set ay [format "%.7e" [lindex $params 6]]
        set by [format "%.7e" [lindex $params 7]]
        set cy [format "%.7e" [lindex $params 8]]
        set yy [format "%.7e" [lindex $params 9]]
        set arc [open [file rootname $catname].tmp w]
        puts $arc "$obj_id $width $ax $bx $cx $xx $ay $by $cy $yy"
        ::close $arc
        return
    }



######################## New method for vmmps ########################
    public method save {} {
        global catname mylist1 mylist2 myresults mode cancel_enter
	global myquad SlitWidthPix SkyRegionPix SpectLen SpectLLoss SpectHLoss BiasType

	global mps_window
	global slit_window
	global rownum_for_newslit catname_for_newslit

	set cancel_enter 0

	#set lastline [exec  tail -1 $catname]
	#set objId [expr [lindex $lastline 0] + 1]

	# AAG - Look for the biggest ID number in catalog
	
        set buffer [exec grep -n "\\--" $catname]
        set headlen [exec echo $buffer | cut -d: -f1]
        set headlen [expr $headlen + 1]
	exec tail +$headlen $catname > $catname\_nohead
	exec tail +$headlen $catname | cut -f1 > $catname\_id
	set fId [::open $catname\_id r ]
	set n 0
	while {[gets $fId line] >= 0} {
	  set n [expr $n + 1]
	  set val($n) [lindex $line 0]
	}
	::close $fId

	# AAG - Now sort in increasing (val(n) is the biggest)

            for {set i 1} {$i<$n} {incr i} {
                set j [expr $i+1]
                if {$val(${j}) < $val(${i})} {
                    set temp $val(${i})
                    set val(${i}) $val(${j})
                    set val(${j}) $temp
                }
            }
            for {set i 1} {$i<$n} {incr i} {
                set j [expr $i+1]
                if {$val(${j}) < $val(${i})} {
                    set temp $val(${i})
                    set val(${i}) $val(${j})
                    set val(${j}) $temp
                }
            }
	set objId [expr $val(${n}) + 1]

	set x_rad 10

	catch {file delete $catname\_nohead}
	catch {file delete $catname\_id}


	if {$mode == "object"} {
	    #puts "DB> Pick Object - Catalog Name: $catname"
	    set heading [exec grep ID $catname | grep X_IMAGE]
	    set headnum [llength $heading]
	    set headnum [expr $headnum - 2]

	    set x [lsearch -exact $heading X_IMAGE]
	    set y [lsearch -exact $heading Y_IMAGE]
	    set xv [lindex $mylist2 0]
	    set yv [lindex $mylist2 1]
	    set mylist $mylist1
	    for {set i 3} {$i < $headnum} {incr i} {
		if {$i == $x} {
		    set mylist $mylist\t$xv
		} else {
		    if {$i == $y} {
			set mylist $mylist\t$yv
		    } else {   
			set mylist $mylist\t0
		    }
		}
	    }

	    set flag R
	    set mylist $mylist\t$x_rad\t$flag

	}
	if {$mode == "slit"} {

	  # AAG - If NewSlit, Look for X_RADIUS in VIMOS Catalog
	  if {$catname_for_newslit != ""} {

	    if {$rownum_for_newslit > 0} { 
	       set buffer_ [exec grep -n "\\--" $catname_for_newslit]
	       set headlen_ [exec echo $buffer_ | cut -d: -f1]
	       set headlen_ [expr $headlen_ + 1]
	       exec tail +$headlen_ $catname_for_newslit > $catname_for_newslit\_nohead

	       set buf ""
	       catch {set buf [exec grep -i ID $catname_for_newslit | grep -i X_RAD]}
	       if {$buf != ""} {
	         set buf [string toupper $buf]
	         set indexID [lsearch -exact $buf ID]
	         set indexALPHA [lsearch -exact $buf ALPHA_VIMOS]
	         set indexDELTA [lsearch -exact $buf DELTA_VIMOS]
	         set indexX [lsearch -exact $buf X_RADIUS]
	         set fId [::open $catname_for_newslit\_nohead r ]
	         set i 0
	         while {[gets $fId line] >= 0} {
		   if {$i == $rownum_for_newslit} {
		      set x_rad [lindex $line $indexX]
		      set objId [lindex $line $indexID] 
		   }
		   set i [expr $i + 1]
    	         }
	         ::close $fId
  	       }
	       catch {file delete $catname_for_newslit\_nohead}

	    } else {
	       #puts "AAG - object is not in catalog"
	    }
	  
	  }

	  #puts "DB> Pick Slit - Catalog Name: $catname"
	  set heading [exec grep ID $catname]
	  set mylist $mylist1\t$mylist2
	  set flag R 
	  set mylist $mylist\t$x_rad\t$flag
	}
	 
	set newrow $objId\t$mylist

        if {[catch {set obj_id [lindex [lindex $myinfo [$myresults.listbox curselection]] 0]} msg]} {
	    #puts "objId: ----- "
        }

	if {$mode == "object"} {
		#puts "DB> New object - from pick : $newrow"
		#set command  "namespace inscope ::cat::vmAstroCat {::.skycat1.vmac1 search}"
		set command  "namespace inscope ::cat::vmAstroCat {::$mps_window search}"
		$myresults cat::vmQueryResult::enter_object $command $newrow

	}
	if {$mode == "slit"} {
	    
	    # AAG - newrow should be now converted to slit

	    #puts "DB> Spoc Info:  Quad:$myquad, $SlitWidthPix, $SpectLen, $SpectLLoss, $SpectHLoss, $SkyRegionPix, $BiasType"
	    set dat_ [open /tmp/vmmps.dat_tmp w]
	    catch {puts $dat_ $newrow}
	    ::close $dat_
	    set arc_ [open /tmp/vmmps_arc w]
	    catch {puts $dat_ ""}
	    ::close $arc_
	    exec vmmps_dim /tmp/vmmps.dat_tmp  /tmp/vmmps_arc /tmp/vmmps.dat
	    catch {file delete /tmp/vmmps.dat_tmp}
	    catch {file delete /tmp/vmmps_arc}
	    file rename -force /tmp/vmmps.dat /tmp/vmmps.dat_tmp
	    exec vmmps_FoV /tmp/vmmps.dat_tmp /tmp/vmmps.dat $myquad
	    file delete /tmp/vmmps.dat_tmp
	    
	    set output [exec vmmps_spoc /tmp/vmmps.dat /tmp/vmmps $SlitWidthPix $SpectLen $SpectLLoss $SpectHLoss $SkyRegionPix 1 $myquad $BiasType]

	    set fId_ [open /tmp/vmmpsM1Q$myquad\.dat r]
	    while {[gets $fId_ line] >= 0} {
	       set id_ [lindex $line 0]
	       set ra_ [lindex $line 1]
	       set dec_ [lindex $line 2]
	       set x_ [lindex $line 3]
	       set y_ [lindex $line 4]
	       set xs_ [lindex $line 5]
	       set ys_ [lindex $line 6]
	       set dx_ [lindex $line 7]
	       set dy_ [lindex $line 8]
	       # AAG - HERE changes for dx & dy
	       set dx_ [format "%.2f" [expr $x_rad + $SkyRegionPix]]
	       set dy_ [format "%.2f" $SlitWidthPix ]
	       set newrow $id_\t$ra_\t$dec_\t$xs_\t$ys_\t$dx_\t$dy_ 
	    }
	    ::close $fId_ 
	    catch {file delete /tmp/vmmpsM1Q$myquad\.dat}
	    catch {file delete /tmp/vmmps.dat}
	    #puts "DB> New slit : $newrow"
	    set command  "namespace inscope ::cat::vmAstroCat {::$slit_window search}"
	    set fId_ [open $catname a]
	    puts $fId_ $newrow
	    ::close $fId_
	    eval $command

	    #$myresults cat::vmQueryResult::enter_object $command $newrow

	}

	if {$cancel_enter==0} {
        	if {$mode == "object"} {
                	EnterObject $mps_window\.er \
                        	-title {Please Modify the data for the object below:} \
                        	-labels $heading \
                        	-values $newrow \
                        	-command [code ::rtd::vmmpsImagePick::edit_row]
		}
        	if {$mode == "slit"} {
                	EnterObject $slit_window\.er \
                        	-title {Please Modify the data for the object below:} \
                        	-labels $heading \
                        	-values $newrow \
                        	-command [code ::rtd::vmmpsImagePick::edit_row_slit]
		}

        }
        close

    }

   public proc edit_row { info } {
     global myresults
     global mps_window
     set command  "namespace inscope ::cat::vmAstroCat {::$mps_window search}"
     $myresults cat::vmQueryResult::enter_object $command $info
   }

   public proc edit_row_slit { info } {
     global slit_window
     global catname
     set command  "namespace inscope ::cat::vmAstroCat {::$slit_window search}"
     
     #puts "JLBQ 101 >> $command $info"
     #puts "DEBUG: JLBQ 1 EnterObject: command = $command, info = $info."                

     if {[string first "vmac2" "$command"] != -1} {
	  # MKI - 2004-05-16
	  # added check for doubles
	 foreach val $info {		     
	     set value [string trimleft [string trimright $val]]	     
	     if {![regexp {^-?[0-9]?[0-9]:[0-9]?[0-9]:[0-9]?[0-9]\.?[0-9]*$} $value]} {
		 if {![regexp {^-?[0-9]+\.?[0-9]*$} $value]} {		
		     error "Values should be integer o doubles ($val)"	       
		 }
	     }
	 }
	 set xdim_x [lindex $info 5]
	 set nxdim_x [expr {int($xdim_x) * 1}]
          #puts "DEBUG: JLBQ xdim_x = $xdim_x"
          if { $nxdim_x > 100 } {
              error_dialog "The value of << DIMX_SLIT >> is great than 100 pixels. Catalogue NOT UPDATED."
              return 1
          }
          set xdim_y [lindex $info 6]
	  set nxdim_y [expr {int($xdim_y) * 1}]
          #puts "DEBUG: JLBQ xdim_y = $xdim_y"
          if { $nxdim_y > 100 } {
               error_dialog "Warning: The value of << DIMY_SLIT >> is great than 100 pixels. It IS WRITTEN into the catalogue, but is your responsibility to know that this value could affect the mask's cut operation."
          }
     }
     set res [regsub -all " " $info \t info]

     set fId_ [open $catname r]
     set nrow 0
     while {[gets $fId_ line] >= 0} {
       set nrow [expr $nrow + 1]
     }
     ::close $fId_

     exec head -[expr $nrow - 1] $catname > $catname\_new
     file rename -force $catname\_new $catname

     set fId_ [open $catname a]
     puts $fId_ $info
     ::close $fId_
     eval $command

   }

###################### Modified for vmmps ############################
    public method pick_object {{parms ""} {wait 0}} {
	if {! $initialized_} {
	    after 1000 [code $this pick_object]
	    return
	}
        set waiting_ $wait
        $itk_component(zoomView) config -command {}
        $itk_component(zoomView) zoom
        $itk_component(zoomView) config -command [code $this scale_changed]
        $w_ configure -cursor cross

        set_values {}

        if {"$parms" == ""} {
            # wait for user to click in image
            set list_ [pick_object_in_image]
        } else {
            lassign $parms imageX_ imageY_ n
            set scale [expr double($itk_option(-maxsize)) / $n]
            set scale [round $scale]
            lassign [$image_ scale] cscale
            $itk_component(zoomView) inc_zoom [expr $scale - $cscale]
            update
        }

        $w_ configure -cursor {}
        $target_image_ zoomview stop

        utilRaiseWindow $w_
#       view
        if { $waiting_ } { return }

        if {[llength $list_] == 10} {
            set_values $list_
        }
    }


    # This method is called to allow the user to pick an object in the main image.
    # The return value is a list of:
    # "ra dec equinox fwhmX fwhmY angle objectPeak meanBackground" 
    # as returned from the rtdimage "statistics" subcommand,
    # or an error.
    
    protected  method pick_object_in_image {} {
        cancel
        set cursor [$target_canvas_ cget -cursor]
        $target_canvas_ configure -cursor cross

        set bindtags [bindtags $target_canvas_]
        set tag pick$w_
        bind $tag <ButtonRelease-1> [code $this picked_object]
        bindtags $target_canvas_ $tag

        global ::$w_.picked
        set $w_.picked {}
        tkwait variable $w_.picked

        bindtags $target_canvas_ $bindtags

        $target_canvas_ configure -cursor $cursor
        set ret [set $w_.picked]
        
        if {"$itk_option(-command)" != ""} {
            eval $itk_option(-command) [list $ret]
        }
        return $ret
    }

    
    # this method is called when the user clicks in the image to select an object
    # or star for the "pick_object" method. 

    public method picked_object {} {
        # display busy cursor in image and pick window...
        $itk_option(-target_image) busy {
            busy {
                if {[catch {set list [$image_ statistics]} msg]} {
                    error_dialog $msg
                    cancel_pick
                } else {
                    picked_wcs_object $list
                }
            }
        }
    }


    # This method can be called when the user has selected an object 
    # or star for the "pick_object" method. 
    # The argument should be the value returned from the rtdimage
    # "statistics" subcommand

    public method picked_wcs_object {retval} {
        global ::$w_.picked
        set $w_.picked $retval
    }


    # callback from isrtdZoomView when the scaling was changed

    protected method scale_changed {} {
        set width [set height $itk_option(-maxsize)]
        $image_ convert dist $width $height canvas nxsize nysize image
        $itk_component(nsize) config -value [format_val $nxsize]
        set_values {}
        cancel 0

        # update the view if fwhm has been calculated
        if {$imageX_ < 1 || $imageY_ < 1} { 
	    return
	}
        $image_ convert dist $width $height canvas nxsize nysize image
        set xc [expr $imageX_-($nxsize/2.0)]
        set yc [expr [$image_ height]-$imageY_-($nysize/2.0)]
        $target_image_ view update $image_ $xc $yc $width $height 0 0 0 0 image
        set list_ {}
        if { $waiting_ } {
	    return
	}
        $itk_option(-target_image) busy {
            busy {
                if {! [catch {set list_ [$image_ statistics]} msg]} {
                    if {[llength $list_] == 10} {
                        set_values $list_ 0
                    }
                }
            }
        }
    }
    
    
    # format a floating point value (which may also be empty)

    protected method format_val {val} {
        if {"$val" == ""} {
            return
        }
        return [format {%.1f} $val]
    }
    

    
    # set the values of the labels from the list (results of "pick_object" call).
    # If list is empty the labels are cleared.
    # If the list is not empty, mark the ra,dec spot in the image.

    protected method set_values {list {with_rmt 1}} {
        lassign $list x y ra dec equinox fwhmX fwhmY angle object background
	global mylist1 mylist2 mode

	# display x,y in chip coords
	if {"$x" != "" && "$y" != ""} {
	    $target_image_ convert coords $x $y image xc yc chip
	} else {
	    set xc $x
	    set yc $y
	}

	$itk_component(x)       config -value [format_val $xc]
        $itk_component(y)       config -value [format_val $yc]
        $itk_component(ra)      config -value $ra
        $itk_component(dec)     config -value $dec
        $itk_component(equinox) config -value $equinox

	set image_x [format_val $xc]
	set image_y [format_val $yc]

	if {$mode == "object"} {
	#   set mylist $ra\t$dec\t$image_x\t$image_y\t$ra\t$dec
	    set mylist1 $ra\t$dec
	    set mylist2 $image_x\t$image_y
	}
        if {$mode == "slit"} {
	    set mylist1 $ra\t$dec
	    set mylist2 $image_x\t$image_y
	}

        $image_ convert dist 0 $itk_option(-maxsize) canvas 0 nsize image
        $itk_component(nsize)      config -value [format_val $nsize]
        $itk_component(object)     config -value [format_val $object]
        $itk_component(background) config -value [format_val $background]

        if {"$fwhmX" == "" || $fwhmX > 0 && $fwhmY > 0} {
            $itk_component(fwhm)   config -value "[format_val $fwhmX] : [format_val $fwhmY]"
            [$itk_component(fwhm)  component entry] config -foreground black
            $itk_component(angle)  config -value [format_val $angle]
        } else {
            $itk_component(fwhm)   config -value "Can't do"
            [$itk_component(fwhm)  component entry] config -foreground red
            $itk_component(angle)  config -value "___oOo___"
        }
        if {"$x" != "" && "$y"} {
            set imageX_ $x
            set imageY_ $y
        }
        if {"$x" != "" && "$y" != "" && $fwhmX < 50 && $fwhmY < 50} {
            #mark_spot $x $y $image_ $canvas_ $angle $fwhmX $fwhmY
            mark_spot $x $y $target_image_ $target_canvas_ $angle $fwhmX $fwhmY 0
            if {$set_result_ && $with_rmt} {
                if {$x < 0 || $y < 0 || $x > [$image_ width] || $y > [$image_ height]} {
                    set_result -1
                } else {
                    set_result "$x $y $fwhmX $fwhmY"
                }
                set set_result_ 0
            }
        }
    }


    # set the set_result_ variable to 1

    protected method set_result_value {} {
        set set_result_ 1
    }


    # mark the given x,y image coordinate point in the given image/canvas with
    # a cross with the given width, height (image pixels) and angle (deg).

    public method mark_spot {imagex imagey image canvas angle width height {blink 0}} {
        # get canvas coords from image coords
        $image convert coords $imagex $imagey image xc yc canvas
        $image convert dist $width $height image w h canvas

        # get bounding box
        set dx [expr $w/2]
        set dy [expr $h/2]

        set x1 [expr $xc+$dx]
        set y1 $yc

        set x2 $xc
        set y2 [expr $yc-$dy]

        set x3 [expr $xc-$dx]
        set y3 $yc

        set x4 $xc
        set y4 [expr $yc+$dy]

        
        # rotate by angle
        # convert to radian
        set rad [expr $angle/57.2958]
        set cosa [expr cos($rad)]
        set sina [expr sin($rad)]

        set rx1 [expr $xc+$cosa*$dx]
        set ry1 [expr $y1-$sina*$dx]

        set rx3 [expr $xc-$cosa*$dx]
        set ry3 [expr $y1+$sina*$dx]

        set rx2 [expr $x2-$sina*$dy]
        set ry2 [expr $yc-$cosa*$dy]

        set rx4 [expr $x2+$sina*$dy]
        set ry4 [expr $yc+$cosa*$dy]

        set bg black
        set fg white
        set tags "mark objects"
        $canvas delete $tags
        
        $canvas create line $rx3 $ry3 $rx1 $ry1 \
            -fill $bg \
            -width 5 \
            -tags "$tags $bg"

        $canvas create line $rx2 $ry2 $rx4 $ry4 \
            -fill $bg \
            -width 5 \
            -tags "$tags $bg"

        $canvas create line $rx3 $ry3 $rx1 $ry1 \
            -fill $fg \
            -width 2 \
            -tags "$tags $fg"

        $canvas create line $rx2 $ry2 $rx4 $ry4 \
            -fill $fg \
            -width 2 \
            -tags "$tags $fg"

        if {$blink} {
            blink_mark $canvas $tags
        }
    }

    
    # This method is used to make the marker given by "tags" blink on
    # and off.
    
    protected method blink_mark {canvas tags {color 0}} {
	catch {after cancel $afterId_}
	set tag [lindex $tags 0]
	if {"[$canvas gettags $tag]" == ""} { return }
	set idx1 $color
	set cols "black white"
	set col1 [lindex $cols $idx1]
	set idx2 [expr ! $color]
	set col2 [lindex $cols $idx2]
	$canvas itemconfigure white -fill $col2
	$canvas itemconfigure black -fill $col1
	set afterId_ [after 500 [code $this blink_mark $canvas $tag $idx2]]
	return
    }

    
    # cancel the current pick operation

######################### Modified for vmmps #########################
    public method cancel {{with_pick 1}} {
        global catname
	$canvas_ delete mark
	$target_canvas_ delete mark
	if {$with_pick} {
	    cancel_pick
	    set imageX_ -1
	    set imageY_ -1
	    set waiting_ 0
	}
    }


    # close this window
    
    public method close {} {
    global catname
        catch {file delete [file rootname $catname].tmp}
        if {$set_result_} {
            set set_result_ 0
            set_result -1
        }
        after idle "wm withdraw $w_"
    }

    # this method is called when the user clicks in the image to select an object
    # or star for the "pick_object" method. In this case, the x,y and ra,dec position 
    # are fixed and only the other info should be calculated (used).

    public method picked_special_object {x y ra dec equinox} {
        # display busy cursor in image and pick window...
        $itk_option(-target_image) busy {
            busy {
                if {[catch {set list [$image_ statistics]} msg]} {
                    error_dialog $msg
                    cancel_pick
                } else {
                    lassign $list {} {} {} {} {} fwhmX fwhmY angle object background
                    picked_wcs_object \
                        [list $x $y $ra $dec $equinox $fwhmX $fwhmY $angle $object $background]
                }
            }
        }
    }
    

    # cancel the wait for the pick_object method and reset the cursor

    public method cancel_pick {} {
        global ::$w_.picked
        global catname
        catch {file delete [file rootname $catname].tmp}
        set $w_.picked ""
    }


    # dummy (called by RtdImage.tcl which assumes that the scaling is propagated)

    public method update_scale {{fx 1} {fy 1}} {
    }


    # returns statistics after image event

    protected method update_now {} {
        global ::$w_.picked
        if [$image_ isclear] { return }
        if {! $waiting_ } {
            if {"[set $w_.picked]" == ""} { return }
        }
        set waiting_ 0
        if {[catch {set list [$image_ statistics]} msg]} {
            return
        }
        $canvas_ delete mark
        $target_canvas_ delete mark
        set_values $list
    }


    # -- options --

    # target (main) RtdImage itcl widget
    itk_option define -target_image target_image Target_image {} {
        set img [cget -target_image]
        set target_image_ [$img get_image]
        set target_canvas_ [$img get_canvas]
    }

    # X shared memory option
    itk_option define -usexshm useXshm UseXshm 1

    # X synchronisation option
    itk_option define -usexsync useXsync UseXsync 1

    # flag: if true, print diagnostic messages
    itk_option define -verbose verbose Verbose {0}

    # font to use for labels
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*

    # font to use for values
    itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*

    # font to use for ra,dec labels (alpha, delta)
    itk_option define -wcsfont wcsFont WcsFont -*-symbol-*-*-*-*-14-*-*-*-*-*-*-*

    # set the width for  displaying labels and values
    itk_option define -labelwidth labelWidth LabelWidth 18

    # set the width for  displaying labels and values
    itk_option define -valuewidth valueWidth ValueWidth 12

    # set the max size of the image sample area (screen dist)
#################### Modified for vmmps 200 -> 300 ####################
    itk_option define -maxsize maxSize MaxSize 300

    # command to evaluate when a selection is made or canceled
    itk_option define -command command Command {}

    # debugging flag
    itk_option define -debug debug Debug 0
    
    # default zoom magnification factor
    itk_option define -factor factor Factor {10} {
        set factor_ [cget -factor]
    }

    # Specify the orientation of image and panel, one of {vertical horizontal}
    itk_option define -orient orient Orient "vertical"


    # -- protected vars -- 
    
    # internal target image
    protected variable target_image_

    # internal target canvas
    protected variable target_canvas_
    
    # internal canvas widget
    protected variable canvas_

    # internal zoomView rtd image
    protected variable image_
    
    protected variable set_result_ 0

    # X coord of last picked object
    protected variable imageX_ -1

    # Y coord of last picked object
    protected variable imageY_ -1

    # output of last statistics command after scaling and pick object command
    protected variable list_

    # id for blink after job
    protected variable afterId_

    # waiting for image event before returning result
    protected variable waiting_ 0

    # set to 1 after init {} was called
    protected variable initialized_ 0
}

