# E.S.O. - VLT project/ ESO Archive
# "@(#) $Id: vmmps_cross_gui.tcl,v 3.24 2004/12/18 04:03:45 vltsccm Exp $"
#
# vmmps_cross_gui.tcl - GUI for SPOC.
#
# See man page vmmps_cross_gui(n) for a complete description.
#
# who          when       what
# ----------   --------- -------------------------------------
# D.Bottini    01 Apr 00 Created

itcl::class cat::vmmps_cross_gui {
    inherit util::TopLevelWidget

    #  constructor

    constructor {args} {
        eval itk_initialize $args

    }

    # destructor

    destructor {
        close
    }
    
    protected method init {} {

	global env catname imaname vimoscat cross_flag
 	#wm title $w_ "vmmps_cross ($itk_option(-number))"
 	wm title $w_ "vmmps_cross"
 	wm iconname $w_ "vmmps_cross ($itk_option(-number))"
        set home $env(VMMPS)
        if {[catch {set mycatname $catname}]} {set mycatname ""}
        if {[catch {set myimaname $imaname}]} {set myimaname ""}
        if {[catch {set myvimoscat $vimoscat}]} {set myvimoscat ""}
        make_layout $mycatname $myimaname $myvimoscat
	set cross_flag 0
	destroy .handhold
    }

    # This method is responsible for the window layout

    protected method make_layout {mycatname myimaname myvimoscat} {

        add_menubar
        add_widgets $mycatname $myimaname $myvimoscat
        add_buttons
        add_progress_bar
        make_short_help
    }


    protected method add_menubar {} {
	TopLevelWidget::add_menubar

	set m [add_menubutton File "Display File menu"]

        #add_menuitem $m command "Select astronomer's catalog" \
	#{Select an astronomer's catalog, only in skycat format} \
	#-command [code $this selastro]

        #add_menuitem $m command "Select VIMOS catalog" \
	#{Select a VIMOS catalog, only X, Y columns (ascii format} \
	#-command [code $this selvimos]

        #add_menuitem $m command "Select VIMOS image" \
	#{Select a VIMOS image} \
	#-command [code $this selima]

	add_menuitem $m command "Close" \
	    {Close this window} \
	    -command [code $this close]
    }

    protected method add_widgets {mycatname myimaname myvimoscat} {

	# dialog widgets
        

	pack [frame $w_.label -borderwidth 2 -relief raised] \
	    -side top -fill x

        pack [LabelEntry $w_.label.astrocat \
                -text "Contributed cat          :" \
                -value $mycatname \
                -relief groove] \
                -side top -anchor e -fill x
        $w_.label.astrocat.entry config -state disabled

        pack [LabelEntry $w_.label.vimoscat \
                -text "VIMOS Pre-image cat:" \
                -value $myvimoscat \
                -relief groove] \
                -side top -anchor e -fill x
        $w_.label.vimoscat.entry config -state disabled

        pack [LabelEntry $w_.label.vimosima \
                -text "VIMOS Pre-image      :" \
                -value $myimaname \
                -relief groove] \
                -side top -anchor e -fill x
        $w_.label.vimosima.entry config -state disabled

        pack [LabelEntry $w_.label.outputcat \
                -text "Output VIMOS cat      :" \
                -value "[file rootname $mycatname]_vm.cat" \
                -relief groove] \
                -side top -anchor e -fill x
        $w_.label.outputcat.entry config -state disabled

	pack [frame $w_.widgets -borderwidth 2 -relief raised] \
	    -side top -fill x

        pack [LabelEntryScale $w_.widgets.tolerance \
                -text "Matching tolerance (arcsec):         " \
                -valuewidth 4 \
                -validate real \
                -relief groove] \
                -side top -anchor e
        $w_.widgets.tolerance.scaleframe.scale config -digit 3
        $w_.widgets.tolerance.scaleframe.scale config -resolution 0.1
        $w_.widgets.tolerance.scaleframe.scale config -from 1.0
        $w_.widgets.tolerance.scaleframe.scale config -to 10.0        
        $w_.widgets.tolerance config -value 5.0
        $w_.widgets.tolerance.entry config -state normal

        pack [LabelEntryScale $w_.widgets.sigma \
                -text "Sigma clipping for residuals:          " \
                -valuewidth 4 \
                -validate integer \
                -relief groove] \
                -side top -anchor e
        $w_.widgets.sigma.scaleframe.scale config -digit 2
        $w_.widgets.sigma.scaleframe.scale config -resolution 0.1
        $w_.widgets.sigma.scaleframe.scale config -from 2.0
        $w_.widgets.sigma.scaleframe.scale config -to 4.0        
        $w_.widgets.sigma config -value 3.0
        $w_.widgets.sigma.entry config -state normal


        pack [LabelEntryScale $w_.widgets.niter \
                -text "Number of iterations:                    " \
                -from 1 \
                -to 10 \
                -value 5 \
                -valuewidth 4 \
                -validate integer \
                -relief groove] \
               -side top -anchor e
        $w_.widgets.niter.entry config -state normal


        frame $w_.frame
        text $w_.frame.03 \
                -width 52 -height 12 \
                -setgrid true -wrap none \
                -xscrollcommand [list $w_.frame.01 set] \
                -yscrollcommand [list $w_.frame.02 set]
        scrollbar $w_.frame.01 \
                -command [list $w_.frame.03 xview] -orient horiz
        scrollbar $w_.frame.02 \
                -command [list $w_.frame.03 yview] -orient vert
        pack $w_.frame.01 -side bottom -fill x
        pack $w_.frame.02 -side right -fill y
        pack $w_.frame.03 -side left -fill both -expand true
        pack $w_.frame -side top -fill both -expand true
    }


    protected method add_buttons {} {
	# dialog buttons
	pack [frame $w_.buttons -borderwidth 2 -relief raised] \
	    -side top -fill x

	pack [button $w_.cross \
	   -text "Run Cross" \
	   -command [code $this cross]] \
	[button $w_.close \
	   -text "Save VIMOS catalog" \
	   -state disabled \
	   -command [code $this close_cross 1]] \
	[button $w_.quit \
	   -text "Quit" \
	   -command [code $this close_cross 0]] \
	   -side left -expand 1 -pady 2m -in $w_.buttons

    }

    protected method add_progress_bar {} {
	# add a progress bar at the botton
	pack [ProgressBar $w_.progress \
	    -length 150] \
	    -side top -fill x
        $w_.progress.entry config -width 10
    }
    
    protected method make_short_help {} {
	TopLevelWidget::make_short_help

	add_short_help $w_.label.astrocat \
	    {path name of user's catalog}	    
	add_short_help $w_.label.vimoscat \
	    {path name of VIMOS Star catalog}	    
	add_short_help $w_.label.vimosima \
	    {path name of VIMOS preparatory image}	    
	add_short_help $w_.label.outputcat \
	    {path name of output VIMOS catalog}	    
	add_short_help $w_.widgets.tolerance \
	    {set catalog matching tolerance in arcsec}
	add_short_help $w_.widgets.niter \
	    {number of matching and fitting iterations}
	add_short_help $w_.widgets.sigma \
	    {set sigma clipping to be applyed on residual rms}
	add_short_help $w_.frame \
	    {output of cross-coordinate program }	    
	add_short_help $w_.cross \
	    {Recoordinate: recoordinate the astronomer's catalog}
	add_short_help $w_.close \
	    {Close: close this window}
        add_short_help $w_.progress \
            {Progress bar: displays status of work in progress}
    }

    public method cross {} {

	global gvar crossfile_tmp crossfile_out cross_flag
	set cross_flag 1
        set Astrocat [$w_.label.astrocat get]
        set VIMOScat [$w_.label.vimoscat get]
        set VIMOSima [$w_.label.vimosima get]
        set Outputcat [$w_.label.outputcat get]

	# AAG - added. Use of Hidden Temporal files
        set TempOut "[file rootname $Outputcat].temp"

        set Tol [$w_.widgets.tolerance get]
        set Niter [$w_.widgets.niter get]
        set Sigma [$w_.widgets.sigma get]

        set Npar 8

        set line ""
        set skycat_file $Astrocat

	$w_.frame.03 delete 1.0 end
	$w_.progress look_busy
	$w_.progress config -text "Running Cross-Catalogue..."
	update idletasks

	if {[file exists $Astrocat] && [file readable $Astrocat]} {
	    set fId [::open $Astrocat r ]
	    set indexCROSS -1
	    set f_header 0
	    while {[gets $fId line] >= 0} {
		if {!$f_header} {
	            set line [string toupper $line]
		    if {[string match *CROSS* $line]} {
			set indexCROSS [lsearch -exact $line CROSS]
			set f_header 1
		    }
		}
	    }
	    ::close $fId
	} else {
	    error_dialog "Warning:$Astrocat: does not exist or has no read permission" .skycat1
	    return -code return
	}
      
	set crosspos [expr $indexCROSS + 1]
	set output [exec vmmps_cross $Astrocat $VIMOScat $VIMOSima $TempOut $Tol $Npar $Sigma $Niter $crosspos 0]
	#puts "DB> Temporal files for cross: "
	#puts "  > Out - $TempOut"

	if {[string match "*Error-Cross*" $output]} {
	    error_dialog "No match found! " .skycat1
	    close
	    }
	$w_.progress reset
	$w_.frame.03 insert end $output
	
	set buffer [exec grep -n "\\--" $Astrocat]
	set headlen [exec echo $buffer | cut -d: -f1]
	# AAG - Index for first line in data section 
	set headlen1 [expr $headlen + 1]       
	# AAG - Index for last line in header section
	set headlen2 [expr $headlen - 2]
	
	exec head -$headlen2 $Astrocat > $TempOut\_head
	exec grep ID $Astrocat | grep RA | cut -f2- > $TempOut\_noid
	exec tail +$headlen1 $Astrocat | cut -f2- > $TempOut\_noid_val
	exec cat  $TempOut\_noid_val >> $TempOut\_noid
	exec paste $TempOut $TempOut\_noid > $TempOut\_nohead
	exec cat $TempOut\_nohead > $TempOut
	
	catch {file delete $TempOut\_head}
	catch {file delete $TempOut\_noid_val}
	catch {file delete $TempOut\_noid}
	catch {file delete $TempOut\_nohead}


	set crossfile_tmp $TempOut
	set crossfile_out $Outputcat

	#AAG - It displays temporal output objets (nmatch in first iteration are marked as red. Those in last iteration are green)"
	plot_nmatch $TempOut "first"
	plot_nmatch $TempOut "last"

	$w_.close config -state active
	
    }


    # plot the guide stars found during cross match  

    public method plot_nmatch { name iter } {

	global tot_first tot_last 
	set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
	set image_ [$skycat get_image]
	set canvas_ [$skycat get_canvas]
	set tags mark$iter

        # it is assumed that image is loaded
        if {"$image_" == ""} {
	    #puts "DB> No image is loaded for ploting"
            return
        }

        # if any objects are selected, deselect them first
	clean_tags $iter

	set equinox [$image_ wcsequinox]
	set w [$image_ dispwidth]
	set h [$image_ dispheight]
	set witdth [$image_ width]
	set height [$image_ height]
        set canvscale [$image_ scale]
	set canvscale [lindex $canvscale 0]
        if {$canvscale<0} {
	  set canvscale [expr -1.0/$canvscale]
         }

	set size [expr 10 * $canvscale]

	set name $name\_$iter
	set line ""
        if {[file exists $name] && [file readable $name]} {
	    #puts "DB> Plotting nmatch object in $name"
            set fId [::open $name r ]
	    set f_header 0
            while {[gets $fId line] >= 0} {
		if {!$f_header} {
		    set line [string toupper $line]
		    if {[string match *X_IMAGE* $line] && [string match *Y_IMAGE* $line] } {
			set indexID [lsearch -exact $line ID]
			set indexX [lsearch -exact $line X_IMAGE]
			set indexY [lsearch -exact $line Y_IMAGE]
                        set f_header 1
		    } 
		} else {
		    set val_id [lindex $line $indexID]
		    set val_x [lindex $line $indexX]
		    set val_y [lindex $line $indexY]

		    if {[catch {$image_ convert coords $val_x $val_y "image equinox" val2_x val2_y canvas}]} {
		   	#puts "DB> Problems while converting coords "
		   	continue
			}

		    #set x [expr $val2_x * $canvscale]
		    #set y [expr $val2_y * $canvscale]
		    set x $val2_x
		    set y $val2_y

		    if {! ($x >= 0 && $y >= 0 && $x < $w && $y < $h)} {
		    	#puts "DB> Out of range (x:$x < w:$w AND y:$y < h:$h)"
		    	continue
		    }
		    set x0 [expr $x-$size]
		    set y0 [expr $y-$size]
		    set x1 [expr $x+$size]
		    set y1 [expr $y+$size]

		    # draw the symbol
		    if {$iter == "first"} {
			set tot_first [expr $tot_first + 1]
			#puts "DB> Plot_nmatch (first iteration). (x: $x, y:$y) "
			draw_circle $canvas_ $x0 $y0 $x1 $y1 red $tags$tot_first
		    } else {
			set tot_last [expr $tot_last + 1]
			#puts "DB> Plot_nmatch (last iteration). (x: $x, y:$y) "
			draw_circle $canvas_ $x0 $y0 $x1 $y1 green $tags$tot_last
		    }			
		    
		}
	    }
	    ::close $fId
	} else {
	    #puts "DB >Warning:$name: does not exist or has no read permission"
	    return
	}   

	catch {file delete $name}
    }


    proc  draw_circle {canvas x0 y0 x1 y1 fg tags} {
	
	$canvas create oval $x0 $y0 $x1 $y1 \
        -outline $fg \
	    -width 2 \
	    -tags $tags
    }

    proc  draw_square {canvas x0 y0 x1 y1 fg tags} {

	$canvas create line $x1 $y0 $x0 $y0 \
	    -fill $fg \
	    -width 2 \
	    -tags $tags 
	$canvas create line $x1 $y1 $x0 $y1 \
	    -fill $fg \
	    -width 2 \
	    -tags $tags 
	$canvas create line $x0 $y0 $x0 $y1 \
	    -fill $fg \
	    -width 2 \
	    -tags $tags 
	$canvas create line $x1 $y0 $x1 $y1 \
	    -fill $fg \
	    -width 2 \
	    -tags $tags 
    }

    proc clean_tags { iter } {
    global tot_first tot_last

        set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
        set image_ [$skycat get_image]
        set canvas_ [$skycat get_canvas]
        set tags mark$iter

        if {$iter=="first"} {set tot $tot_first}
        if {$iter=="last"} {set tot $tot_last}
        for {set n 1} {$n <= $tot} {incr n 1} {
                $canvas_ delete $tags$n
        }
        $canvas_ delete grip
        update idletasks

    }

    public method selastro {} {
    set mycatname [filename_dialog [pwd] *.cat .skycat1]
    $w_.label.astrocat configure -value $mycatname
}

    public method selvimos {} {
    set myvimoscat [filename_dialog [pwd] *.txt .skycat1]
    $w_.label.vimoscat configure -value $myvimoscat
}

    public method selima {} {
    set myvimosima [filename_dialog [pwd] *.fits .skycat1]
    $w_.label.vimosima configure -value $myvimosima
}

public proc vmmpsWaitDialog {dialog} {
    global gvar

    set gvar(${dialog}_vmmpsExit) -1
    vwait gvar(${dialog}_vmmpsExit)
    wm withdraw $gvar(${dialog}Wdg)
    return $gvar(${dialog}_vmmpsExit)
}


public proc vmmpsEnvokeDialog {dialog userText} {
    global gvar env
    set build 1

    # Catch the case where window has been distroyed by wm close
    if {[info exists  gvar(${dialog}Wdg)] } {
        if {[winfo exists $gvar(${dialog}Wdg)]} {set build 0}
    }
    set gvar(${dialog}_exit) 0
    if {$build} {
        set gvar(${dialog}Wdg) [istClassDialog \
                ${dialog}_uifClass $dialog $userText]
    } else {
        wm deico $gvar(${dialog}Wdg)
    }
}

    public method close_cross { save } {
	global crossfile_tmp crossfile_out cross_flag

	set skycat "namespace inscope ::skycat::SkyCat .skycat1.image"
	set image_ [$skycat get_image]
	set canvas_ [$skycat get_canvas]

	if {$save} { 

	    #	AAG -
	    #       Read Temp Out cat and search for CROSS column. If found
	    #       read each column value. If cross_value = 1 leave line
	    #       but remove CROSS field.


	    if {[file exists $crossfile_tmp] && [file readable $crossfile_tmp]} {
		#puts "DB> Opening $crossfile_tmp for reading"
		set fId [::open $crossfile_tmp r ]
		set indexCROSS -1
		set val_cat ""
		set f_header 0
		while {[gets $fId line] >= 0} {
		    if {!$f_header} {
			set line [string toupper $line]
			if {[string match *CROSS* $line]} {
			    set indexCROSS [lsearch -exact $line CROSS]
			    set line [lreplace $line $indexCROSS $indexCROSS]
			    set f_header 1
		    	}
			set res [regsub -all " " $line \t line]
			set val_cat $val_cat$line\n
		    } else {
			
			if {$indexCROSS != -1} {
			    set val_cross [lindex $line $indexCROSS]
			    set line [lreplace $line $indexCROSS $indexCROSS]
			    set res [regsub -all " " $line \t line]
		            set val_cat $val_cat$line\n
			} else {
			    set res [regsub -all " " $line \t line]
			    set val_cat $val_cat$line\n
			}
			
		    }
		}
		::close $fId
	    } else {
		error_dialog "Warning:$crossfile_tmp: does not exist or has no read permission" .skycat1
		return -code return
	    }
	    
	    set fileId [::open $crossfile_tmp\_ w ]
	    puts -nonewline $fileId $val_cat
	    ::close $fileId
	    
	    # AAG - This Temporal file should be removed now
	    catch {file rename -force $crossfile_tmp\_ $crossfile_out}
	    
	    #exec cp $crossfile_tmp $crossfile_out
	    #puts "DB> Output catalog from Cross: $crossfile_out"
	}
	if {$cross_flag} {
	    #puts "DB> Deleting Temporal file $crossfile_tmp"
	    catch {file delete $crossfile_tmp}	
	    set folder [file rootname $crossfile_out]
	    set i [string last / $folder]
	    if {$i != -1} {
		set folder [string range $folder 0 [ expr $i - 1 ] ]
	    }
	    catch {eval "file delete [glob -nocomplain $folder/*.tmp]"}
	    #puts "DB> $folder/*.tmp removed"

	    clean_tags "first"
	    clean_tags "last"

	}


	.skycat1.image clear
	close

   }

        # close this window
    public method close {} {
        destroy $w_ 
	#destroy .handhold
   }



#   protected common astrocat_ [astrocat ::cat::.vmastrocat]

   
}
