# E.S.O. - VLT project/ ESO Archive
# "@(#) $Id: vmmps_spoc_gui.tcl,v 3.24 2004/12/18 04:03:45 vltsccm Exp $"
#
# vmmps_spoc_gui.tcl - GUI for SPOC.
#
# See man page vmmps_spoc_gui(n) for a complete description.
#
# who          when       what
# ----------   --------- -------------------------------------
# D.Bottini    01 Apr 00 Created
# jbrancac     25/02/04  Change the minimum width of the slit from 0.8 to 0.6. 
#

itcl::class cat::vmmps_spoc_gui {
    inherit util::TopLevelWidget

    #  constructor

    constructor {args} {
        eval itk_initialize $args
    }

    # destructor

    destructor {
        close
    }
    
    protected method init {} {

        global catname quad env myquad
	global adpquad catadp
        set myquad $quad
        set mycatname $catname
	set adpquad ""
	set catadp ""
 	wm title $w_ "vmmps_spoc ($itk_option(-number))"
 	wm iconname $w_ "vmmps_spoc ($itk_option(-number))"
        set home $env(VMMPS)
	set grm [open $home/config/vmmps_grism.cfg r]
	set buf_grism ""
        while {[gets $grm line] >= 0} {
          if {[string match *##* $line] == 0} { lappend buf_grism $line}
        }
        ::close $grm

        set ngrism [llength $buf_grism]
	make_layout $mycatname $myquad
    }

    # This method is responsible for the window layout

    protected method make_layout {mycatname myquad} {

        add_menubar $mycatname $myquad
        add_widgets $mycatname $myquad
        add_buttons $mycatname $myquad
        add_progress_bar
        make_short_help
    }


    protected method add_menubar {mycatname myquad} {
	TopLevelWidget::add_menubar

	set m [add_menubutton File "Display File menu"]

	add_menuitem $m command "Close" \
	    {Close this window} \
	    -command [code $this cancel $mycatname $myquad]
		
    }

    protected method add_widgets {mycatname myquad} {

	# dialog widgets
	global xx
        
	pack [frame $w_.option -borderwidth 2 -relief raised] \
	    -side top -fill x

        pack [LabelChoice $w_.option.bias \
                -text "Code Method:" \
                -value Max_Opt \
                -relief groove \
                -orient Vertical \
                -choice {Max_Opt Normal_Opt}] \
                -side left -anchor w

        pack [LabelMenu $w_.option.grism \
                -text "Grism:" \
                -relief groove \
                -orient horizontal] \
                -side top -anchor ne

        for {set n 0} {$n < $ngrism} {incr n 1} {
          $w_.option.grism add -label "[lindex [lindex $buf_grism $n] 0]" -command [code $this spectCut]
        }

	pack [frame $w_.widgets -borderwidth 2 -relief raised] \
	    -side top -fill x

        pack [LabelEntryScale $w_.widgets.width \
                -text "Slit Width (arcsec):" \
                -valuewidth 4 \
                -validate real \
                -relief groove] \
                -side top -anchor e

        $w_.widgets.width.scaleframe.scale config -digit 3
        $w_.widgets.width.scaleframe.scale config -resolution 0.05
# JLBQ  $w_.widgets.width.scaleframe.scale config -from 0.8
        $w_.widgets.width.scaleframe.scale config -from 0.6
        $w_.widgets.width.scaleframe.scale config -to 5.0        
        $w_.widgets.width config -value 1.0
        $w_.widgets.width.entry config -state normal 

        pack [LabelEntryScale $w_.widgets.sky \
                -text "Sky Region (arcsec):" \
                -valuewidth 4 \
                -validate real \
                -relief groove] \
                -side top -anchor e

        $w_.widgets.sky.scaleframe.scale config -digit 3
        $w_.widgets.sky.scaleframe.scale config -resolution 0.2
        $w_.widgets.sky.scaleframe.scale config -from 0.2
        $w_.widgets.sky.scaleframe.scale config -to 10.0        
        $w_.widgets.sky config -value 1.8
        $w_.widgets.sky.entry config -state normal 

        pack [LabelEntryScale $w_.widgets.lloss \
                -text "Low Cut (Angstrom):" \
                -valuewidth 4 \
                -validate integer \
                -relief groove] \
                -side top -anchor e

        pack [LabelEntryScale $w_.widgets.hloss \
                -text "High Cut (Angstrom):" \
                -valuewidth 4 \
                -validate integer \
                -relief groove] \
                -side top -anchor e

        spectCut

        pack [LabelEntryScale $w_.widgets.mask \
                -text "Number of Masks:" \
                -from 1 \
                -to 15 \
                -value 1 \
                -valuewidth 4 \
                -validate integer \
                -relief groove] \
               -side top -anchor e

	pack [frame $w_.label -borderwidth 2 -relief raised] \
	    -side top -fill x

        pack [LabelMessage $w_.label.catalog \
                -text "Input Catalog:" \
                -value [file tail $mycatname] \
                -relief groove] \
                -side top -anchor e -fill x

	label  $w_.adplabel -text "ADP Main Name:"\
	    -font -Adobe-Times-Medium-R-Bold-*-15-*
	entry $w_.adpname -width 5 -relief sunken -textvariable xx
	pack $w_.adplabel $w_.adpname -side top -anchor e -fill x


	set path [file rootname $mycatname]
	set i [string last / $path]
	set relname [string range $path [expr $i + 1] end]
	set base [string range $path 0 $i]
	#puts "DB> Relative Current ADP name : $relname"

	set xx $relname

        frame $w_.frame
        text $w_.frame.03 \
                -width 52 -height 15 \
                -setgrid true -wrap none \
                -xscrollcommand [list $w_.frame.01 set] \
                -yscrollcommand [list $w_.frame.02 set]
        scrollbar $w_.frame.01 \
                -command [list $w_.frame.03 xview] -orient horiz
        scrollbar $w_.frame.02 \
                -command [list $w_.frame.03 yview] -orient vert
        pack $w_.frame.01 -side bottom -fill x
        pack $w_.frame.02 -side right -fill y
        pack $w_.frame.03 -side left -fill both -expand true
        pack $w_.frame -side top -fill both -expand true
    }


    protected method add_buttons {mycatname myquad} {
	# dialog buttons
	pack [frame $w_.buttons -borderwidth 2 -relief raised] \
	    -side top -fill x

	pack [button $w_.spoc \
	   -text "Assign Slits" \
	   -command [code $this spoc $mycatname $myquad]] \
	    [button $w_.close \
	   -text "Back to Changing FLAGS" \
	   -command [code $this cancel $mycatname $myquad]] \
	    [button $w_.exit \
		 -text "Close and Save" \
		 -state disabled \
		 -command [code $this save_spoc]] \
	   -side left -expand 1 -pady 2m -in $w_.buttons

    }

    protected method add_progress_bar {} {
	# add a progress bar at the botton
	pack [ProgressBar $w_.progress \
	    -length 150] \
	    -side top -fill x
        $w_.progress.entry config -width 10
    }
    
    protected method make_short_help {} {
	TopLevelWidget::make_short_help

	add_short_help $w_.widgets.width \
	    {set slit width (arcsec)}
	add_short_help $w_.widgets.sky \
	    {set minimum sky reagion per side (arcsec)}
	add_short_help $w_.widgets.lloss \
	    {set low cut for spectrum range (angstrom)}
	add_short_help $w_.widgets.hloss \
	    {set high cut for spectrum range (angstrom)}
	add_short_help $w_.widgets.mask \
	    {set number of ADP (masks) to be generate for the selected catalog}
	add_short_help $w_.option.grism \
	    {set grism to be used for the observation}
	add_short_help $w_.option.bias \
	    {Max_Opt=max N. of objects, Normal_Opt=don't change object dimension distribution}
	add_short_help $w_.label.catalog \
	    {name of input catalog}	    
	add_short_help $w_.frame \
	    {output of the Slit Positioning Optimization Code}	    
	add_short_help $w_.spoc \
	    {Make ADP: generate the slit catalog}
	add_short_help $w_.close \
	    {Close: close this window}
        add_short_help $w_.progress \
            {Progress bar: displays status of work in progress}
    }

      protected method spectCut {} {
	  set Grism [string trim [lindex [$w_.option.grism get] 0] ,]
	  for {set n 0} {$n < $ngrism} {incr n 1} {
	      if {$Grism == "[string trim [lindex [lindex [lindex $buf_grism $n] 0] 0] ,]"} {
		  set SpectLen [lindex [lindex $buf_grism $n] 1]
		  set SpectDisp [lindex [lindex $buf_grism $n] 2]
		  set SpectL [lindex [lindex $buf_grism $n] 3]
		  set SpectH [lindex [lindex $buf_grism $n] 4]
		  set Spect0 [lindex [lindex $buf_grism $n] 5]
	      }
	  }
	  $w_.widgets.lloss configure -from [expr $SpectL + 50]
	  $w_.widgets.lloss configure -to $Spect0
	  $w_.widgets.lloss configure -value [expr $SpectL + 50]
	  $w_.widgets.hloss configure -from $Spect0
	  $w_.widgets.hloss configure -to [expr $SpectH - 50]
	  $w_.widgets.hloss configure -value [expr $SpectH - 50]
      }
                      
      protected method spoc {mycatname myquad} {
	global catname_for_newslit
	
	global xx MaskNum_
	global adpquad
        global SlitWidthPix SkyRegionPix SpectLen SpectLLoss SpectHLoss BiasType 
       
	set catname_for_newslit $mycatname

        set Grism [string trim [lindex [$w_.option.grism get] 0] ,]
        for {set n 0} {$n < $ngrism} {incr n 1} {
	    if {$Grism == "[string trim [lindex [lindex [lindex $buf_grism $n] 0] 0] ,]"} {
            set SpectLen [lindex [lindex $buf_grism $n] 1]
            set SpectDisp [lindex [lindex $buf_grism $n] 2]
            set SpectL [lindex [lindex $buf_grism $n] 3]
            set SpectH [lindex [lindex $buf_grism $n] 4] 
            set Spect0 [lindex [lindex $buf_grism $n] 5]
          }
        }

        set SlitWidth [$w_.widgets.width get]
        set SkyRegion [$w_.widgets.sky get]
        set SpectLVal [$w_.widgets.lloss get]
        set SpectHVal [$w_.widgets.hloss get]
        set MaskNum [$w_.widgets.mask get]
	set MaskNum_ $MaskNum
        set Bias [$w_.option.bias get]

        set SpectLLoss [expr ($SpectLVal - $SpectL) / $SpectDisp]
        set SpectHLoss [expr ($SpectH - $SpectHVal) / $SpectDisp]
        
        if {$Bias == "Max_Opt"} {set BiasType "M"
           } elseif {$Bias == "Normal_Opt"} {set BiasType "N"}

        set SlitWidthPix [expr $SlitWidth / 0.205]
        set SkyRegionPix [expr $SkyRegion / 0.205]

        $w_.progress look_busy
	$w_.progress config -text "Running SPOC..."
	update idletasks


####################
	# AAG - ADP File should not be saved yet. Work only with temporal
	# 1. Convert Temporal in cat file. Load it in skycat
        # 2. Allow changes through panel
	# 3. Make ADP

	set path [file rootname $mycatname]
	set i [string last / $path]
	set relname [string range $path [expr $i + 1] end]
	set base [string range $path 0 $i]

	set adpname [$w_.adpname get]

	set adpname $base$adpname
	#puts "DB> Current Absolute ADP name: $adpname"

#	set adpquad $adpname\Q$myquad
	set adpquad $adpname\_$Grism\_

        set file2del [glob -nocomplain [file rootname $adpquad]\M*]
	if {"$file2del" != ""} {
          if {![confirm_dialog "The following files exist:\n $file2del \n Do you want to overwrite them ?" $w_]} {
	    #puts "DB> Files not removed"
	    $w_.progress reset
	    #$w_.frame.03 delete 1.0 end
            return
          }
        }

        catch {eval "file delete [glob -nocomplain [file rootname $adpquad]\M*]"}

##### 


	file rename -force [file rootname $mycatname].dat [file rootname $mycatname].dat_temp
#	exec vmmps_FoV [file rootname $mycatname].dat_temp [file rootname $mycatname].dat $myquad
	exec vmmps_FoV [file rootname $mycatname].dat_temp [file rootname $mycatname].dat $myquad
	file delete [file rootname $mycatname].dat_temp
	#puts "DB> Spoc Info:  Quad:$myquad, $SlitWidthPix, $SpectLen, $SpectLLoss, $SpectHLoss, $SkyRegionPix, $BiasType"
#        set output [exec vmmps_spoc [file rootname $mycatname].dat [file rootname $mycatname]Q$myquad \
#        $SlitWidthPix $SpectLen $SpectLLoss $SpectHLoss $SkyRegionPix $MaskNum $BiasType]
        set output [exec vmmps_spoc [file rootname $mycatname].dat $adpquad \
			$SlitWidthPix $SpectLen $SpectLLoss $SpectHLoss $SkyRegionPix $MaskNum $myquad $BiasType]

        $w_.progress reset
        $w_.frame.03 delete 1.0 end
        $w_.frame.03 insert end $output

        set buffer [exec grep #fits $mycatname | cut -f2- -d:]
	set cfg [open [file rootname $mycatname].cfg w]
	puts $cfg "$buffer"
        ::close $cfg

        set dat [open [file rootname $mycatname].arc w]
        catch {puts $dat [exec grep #arc $mycatname | cut -f2- -d:]}
        ::close $dat
        
        set log [open [file rootname $mycatname].log w]
        
        puts $log "########################## SPOC LOG ##########################"
        puts $log "--------------------------------------------------------------"
        puts $log "Input parameters of SPOC"
        puts $log "--------------------------------------------------------------"
        puts $log "Input catalog name: [file rootname $mycatname].dat"
        puts $log "Slit catalog name: [file rootname $adpquad]M<n>Q<quad>.adp"
        puts $log "Slit width (arcsec): $SlitWidth" 
        puts $log "Grism: $Grism"
        puts $log "Lower Spectrum Loss (pix): $SpectLLoss"
        puts $log "Higher Spectrum Loss (pix): $SpectHLoss"
        puts $log "SpectLVal: $SpectLVal"
        puts $log "SpectHVal: $SpectHVal"
        puts $log "Sky region (arcsec): $SkyRegion"
        puts $log "Number of masks: $MaskNum"
        puts $log "SPOC mode: $Bias"
        puts $log "--------------------------------------------------------------"
        puts $log "OUTPUT OF SPOC"
        puts $log "--------------------------------------------------------------"
        puts $log "$output"
        puts $log "--------------------------------------------------------------"
        ::close $log
       

#       exec vmmps_spoc2paf [file rootname $mycatname] [file rootname $mycatname]Q$myquad $MaskNum $Grism $SpectLen $SkyRegion
#        exec vmmps_spoc2paf [file rootname $mycatname] $adpquad $MaskNum $Grism $SpectLen $SkyRegion



	#puts ">>> DB JLBQ > vmmps_spoc2paf [file rootname $mycatname] $adpquad $MaskNum $Grism $SpectLen $SkyRegion"

        exec vmmps_spoc2paf [file rootname $mycatname] $adpquad $MaskNum $Grism $SpectLen $SkyRegion

        catch {file delete [file rootname $mycatname].cfg}
        catch {file delete [file rootname $mycatname].arc}

	for {set n 1} {$n <= $MaskNum} {incr n 1} {
#          set datname [file rootname $mycatname]Q$myquad\M$n.dat
           set datname [file rootname $adpquad]M$n\Q$myquad.dat
           catch {file delete $datname}
        }
	$w_.exit config -state active

    }

  public method save_spoc {} {
      global adpquad
      global myquad
      global Ncur
      global mps_window
      if {$mps_window != ""} {
	  #puts "DB> Closing MPS window $mps_window"
	  $mps_window close
      }
      if {$adpquad!=""} {
	  set id "namespace inscope ::skycat::SkyCat .skycat1.image"
	  set classname "::skycat::vmSkySearch"
	  set debug 0
	  set catadp $adpquad\M1\Q$myquad.adp
	  cat::vmAstroCat::conv_from_adp $catadp
	  set catname $catadp\_cat
	  #puts "DB> Loading catalog : $catname"
	  set Ncur 1
	  cat::vmAstroCat::new_catalog $catname $id $classname $debug [cat::vmAstroCat::is_tcs_catalog $catname] "catalog"
	  #cat::vmAstroCat::slits
      }
      close
   }


proc promptDialog {title text} {

  global ::button
  global xx 

# 1. Create top level window and divide it in top center and bottom parts
  toplevel .d -class Dialog
  wm title .d $title
  wm iconname .d Dialog
  frame .d.top -relief raised -bd 1
  pack  .d.top -side top -fill both

  frame .d.cent -relief raised -bd 1
  pack .d.cent -side top -fill both

  frame .d.bot -relief raised -bd 1
  pack  .d.bot -side bottom -fill both

# 2. fill top part with message
  message .d.top.msg -width 5i -text $text\
    -font -Adobe-Times-Medium-R-Normal-*-180-*
  pack .d.top.msg -side right -expand 1 -fill both\
    -padx 3m -pady 3m

# 3. Fill central part with 1 entry
  label .d.cent.label1 -text "ADP"\
      -font -Adobe-Times-Medium-R-Normal-*-180-*
  entry .d.cent.entrx -width 20 -relief sunken -textvariable xx
  pack  .d.cent.label1 .d.cent.entrx -side \
      left -padx 1m -pady 2m


# 4. Fill bottom part with ABORT button
  button .d.bot.button0 -text CANCEL -command {
    set button 1
  }
  pack  .d.bot.button0 -in .d.bot -side bottom -padx 2m -pady 2m\
    -ipadx 2m -ipady 1m -fill both

  button .d.bot.button1 -text OK -command {
    set button 2
  }

  pack  .d.bot.button1 -in .d.bot -side bottom -padx 2m -pady 2m\
    -ipadx 2m -ipady 1m -fill both

  tkwait variable button
  destroy .d
  set ret "$xx"
  return $button,$ret

}
    public method cancel {mycatname myquad} {
        catch {file delete [file rootname $mycatname].dat}
        close
   }

    public proc getNmask {} {
    global MaskNum_
    return $MaskNum_
   }


    # close this window

    public method close {} {
        destroy $w_ 
   }

   protected common ngrism
   protected common buf_grism
}
