#!/bin/sh

## Installation script for the VMMPS tool

UNPACK_DIR=`pwd`

if [ ! -f "$UNPACK_DIR/vmmps_install.sh" ] ; then
    echo "Please run this installation script from inside the package"
    echo "directory (cd <some_path>/vmmps-3.24-LinuxRH9/)"
    exit;
fi

## Welcome banner and warning
echo "** Welcome to the VMMPS installation script **\n"

echo "======================================================"
echo "PLEASE NOTE: this package is to be used for period 75"
echo "and later of observations with the VIMOS instrument."
echo "======================================================\n"


## Choice of installation directory

echo "1) Installation directory:"
echo "--------------------------"

INSTALL_PATH=$UNPACK_DIR

echo "Where do you want to install the VMMPS package ? The default is"
echo "the current directory."
echo ""
echo "WARNING: binaries and other files will be installed in"
echo " - <install_dir>/bin"
echo " - <install_dir>/lib"
echo " - <install_dir>/config"
echo ""
echo "...thus overwriting any previous installation of VMMPS and SKYCAT !\n"

repeatStep=1

while [ $repeatStep -eq 1 ]
do
    echo "Please enter the installation directory [" $INSTALL_PATH "] :"

    read answer

    test $answer
    if [ $? -ne 0 ] ; then
	repeatStep=0
    else

	GREP=`echo $answer | grep "^/"`

	if [ $? -ne 0 ] ; then

	    echo "Please enter an absolute path (starting with / )."

	elif [ ! -d $answer ] ; then

	    echo $answer "does not appear to exist. Do you want to create it ? [y]"
	    read makedir
	    test $makedir
	    if [ $? -ne 0 ] ; then
		makedir=y
	    fi

	    if [ $makedir = "y" ] ; then
		mkdir -p $answer 2>/dev/null
		if [ $? -ne 0 ] ; then
		    echo "Cannot create this directory ! Please choose another one...\n"
		else
		    INSTALL_PATH=$answer
		    repeatStep=0
		fi
	    fi

	elif [ ! -w $answer ] ; then
	    echo "You do not appear to have write access to" $answer "."
	    echo "Please choose another directory for installation."
	else
	    INSTALL_PATH=$answer
	    repeatStep=0
	fi
    fi
done

echo "Installing VMMPS package in "$INSTALL_PATH"\n"

echo
echo "3) Installing the VMMPS package"
echo "-------------------------------\n"


echo "done.\n"

echo "Installing scripts and binaries... "

BINDIR=$INSTALL_PATH/bin
mkdir -p $BINDIR

VMMPSPATH=`echo $INSTALL_PATH | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`

sed -e s/@@VMMPSBASEPATH@@/$VMMPSPATH/ bin/vmmps.tpl | sed s/\\\\//g > $BINDIR/vmmps
chmod u+x $BINDIR/vmmps

if [ $INSTALL_PATH != $UNPACK_DIR ] ; then
    echo "(this may take a while) "
    cp bin/* $BINDIR/.
    cp -pr skycat.app $BINDIR/skycat.app
    ln -s $BINDIR/skycat.app/Contents/Resources/bin/skycat $BINDIR/skycat

    chmod u+x $BINDIR/*
    
    mkdir -p $INSTALL_PATH/config
    cp -r config/* $INSTALL_PATH/config/.

    mkdir -p $INSTALL_PATH/lib
    cp -r lib/* $INSTALL_PATH/lib/.

    mkdir -p $INSTALL_PATH/LOGS

    mkdir -p $INSTALL_PATH/test

    mkdir -p $INSTALL_PATH/template

fi

echo "done."

echo "Cleaning up..."

if [ $INSTALL_PATH != $UNPACK_DIR ] ; then
    CLEANDIR=1
    rm -rf $UNPACK_DIR/*
else
    CLEANDIR=0
    rm -f bin/vmmps.tpl
    rm -f vmmps_install.sh
fi

echo "done"
echo

if [ $CLEANDIR -eq 1 ] ; then
    echo "(NB: you can remove the directory" $UNPACK_DIR ")\n"
fi

echo "** VMMPS was built successfully **\n"

echo "The VMMPS tool can be launched with the command"
echo $INSTALL_PATH"/bin/vmmps"
echo


